#!/usr/bin/perl -w

# Copyright 2014, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Print a tikz picture showing how the SB tree row regions copy out under
# left and right matrices, with dashed parallelograms in between.

use 5.004;
use strict;
use Math::Matrix;
use Math::PlanePath::RationalsTree;

use FindBin;
use lib $FindBin::Bin;
use MyRationals 'SB_left','SB_right','mvec_to_pq','mvec_to_str';
use MyPrintwrap ':all';


# Region printed is between depth and depth+1 copied out 3 times and which
# therefore is portions of depth+1 to depth+2.
#
my $depth = 3;


print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyD}{$depth}\n";

my $path = Math::PlanePath::RationalsTree->new(tree_type => 'SB');
my @vecs;
foreach my $n (($path->tree_depth_to_n($depth+1)
                .. $path->tree_depth_to_n_end($depth+1)),
               reverse($path->tree_depth_to_n($depth)
                       .. $path->tree_depth_to_n_end($depth))) {
  my ($x,$y) = $path->n_to_xy($n);
  push @vecs, Math::Matrix->new([$x],[$y]);
}

foreach my $m (SB_left(), SB_right()) {
  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap += 2;
  foreach my $i (0 .. $#vecs) {
    my $v = $vecs[$i];
    my $join = ($i == 0 ? '' : "--");
    my $str = mvec_to_str($m*$v);
    printwrap("$join($str)");
  }
  printwrap("--cycle;\n");
}

foreach my $d_offset (0,1) {
  my $n = $path->tree_depth_to_n_end($depth + $d_offset);
  my ($x,$y) = $path->n_to_xy($n);
  my $v = Math::Matrix->new([$x],[$y]);
  my $lv = SB_left() * $v;
  ($x,$y) = mvec_to_pq($lv);
  print "  \\coordinate (L-across-$d_offset) at ($x,$y);\n";
  print "  \\coordinate (R-across-$d_offset) at ($y,$x);\n";
}

print "  % end generated\n";
exit 0;
