#!/usr/bin/perl -w

# Copyright 2013, 2014, 2017, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::RationalsTree;
use Math::PlanePath::FractionsTree;

use FindBin;
use lib $FindBin::Bin;
use MyRationals 'Identity_2x2','SB_left','SB_right';

my $depth_limit = 2;

print "  % generated by tools/$FindBin::Script\n";

sub matrix_str_tex {
  my ($m) = @_;
  return join('\\\\', map {join('&', @{$m->[$_]})} 0 .. $#$m);
}
sub matrix_str_pmatrix {
  my ($m) = @_;
  return "\\begin{pmatrix} " . matrix_str_tex($m) . " \\end{pmatrix}";
}
sub matrix_str_gp {
  my ($m) = @_;
  return '[' . join(';', map {join(',', @{$m->[$_]})} 0 .. $#$m) . ']';
}


my $descend;
my @rows;
$descend = sub {
  my ($m,$depth) = @_;
  my $indent = '  'x($depth+1);

  push @{$rows[$depth]}, matrix_str_gp($m);

  my $str = ($depth <= 4 ? matrix_str_pmatrix($m) : "");
  print $indent;
  if ($depth == 0) { print '\\'; }
  print "node {\$$str\$}\n";
  return if $depth > $depth_limit;

  foreach my $LR (SB_left(), SB_right()) {
    print "${indent}child {\n";
    $descend->($LR*$m, $depth+1);
    print "${indent}}\n";
  }
};
$descend->(Identity_2x2(), 0);
print "  ;\n";
foreach my $depth (0 .. $#rows) {
  print "% GP-Test  SB_row_matrices($depth) == [",
    join(',',@{$rows[$depth]}),
    "]\n";
}
print "  % end generated\n";
print "\n";

exit 0;
